package com.zeocollect;

import android.util.Log;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.zeotap.collect.Collect;
import com.zeotap.collect.CollectOptions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

@ReactModule(name = ZeoCollectModule.NAME)
public class ZeoCollectModule extends ReactContextBaseJavaModule {
    public static final String NAME = "ZeoCollect";
    CollectOptions.Builder builder;
    Collect collect;
    ReactApplicationContext mReactContext;

    public ZeoCollectModule(ReactApplicationContext reactContext) {
        super(reactContext);
        mReactContext = reactContext;
    }

    @ReactMethod
    public void initalizeBySettingOptions(ReadableMap rawOptions) {
        builder = CollectOptions.builder(mReactContext);
        HashMap<String, Object> clientOptions = rawOptions.toHashMap();

        if (clientOptions.get("android_write_key") != null && !Objects.equals(clientOptions.get("android_write_key"), "")
                && clientOptions.get("android_write_key") instanceof String) {
            builder = builder.credential((String) clientOptions.get("android_write_key"));
        }
        if (clientOptions.get("logging") != null && clientOptions.get("logging") instanceof Boolean) {
            builder = builder.enableLogging((Boolean) clientOptions.get("logging"));
        }
        if (clientOptions.get("batch_size") != null && !Objects.equals(clientOptions.get("batch_size"), "")
                && (clientOptions.get("batch_size") instanceof Integer || clientOptions.get("batch_size") instanceof Double)) {
            if (clientOptions.get("batch_size") instanceof Double) {
                builder = builder.uploadBatchSize(((Double) clientOptions.get("batch_size")).intValue());
            } else if (clientOptions.get("batch_size") instanceof Integer) {
                builder = builder.uploadBatchSize((Integer) clientOptions.get("batch_size"));
            }
        }
        if (clientOptions.get("optout") != null && clientOptions.get("optout") instanceof Boolean) {
            builder = builder.optOut((Boolean) clientOptions.get("optout"));
        }
        if (clientOptions.get("service_interval") != null && !Objects.equals(clientOptions.get("service_interval"), "")
                && (clientOptions.get("service_interval") instanceof Integer || clientOptions.get("service_interval") instanceof Double)) {
            if (clientOptions.get("service_interval") instanceof Double) {
                builder = builder.serviceUploadInterval(((Double) clientOptions.get("service_interval")).intValue());
            } else if (clientOptions.get("service_interval") instanceof Integer) {
                builder = builder.serviceUploadInterval((Integer) clientOptions.get("service_interval"));
            }
        }
        if (clientOptions.get("useConsent") != null && clientOptions.get("useConsent") instanceof Boolean) {
            builder = builder.useConsent((Boolean) clientOptions.get("useConsent"));
        }
        if (clientOptions.get("maxCacheSize") != null && !Objects.equals(clientOptions.get("maxCacheSize"), "") &&
                (clientOptions.get("maxCacheSize") instanceof Integer || clientOptions.get("maxCacheSize") instanceof Double)) {
            if(clientOptions.get("maxCacheSize") instanceof Double){
                builder = builder.maxCacheSize(((Double) clientOptions.get("maxCacheSize")).intValue());
            } else if (clientOptions.get("maxCacheSize") instanceof Integer){
                builder = builder.maxCacheSize((Integer) clientOptions.get("maxCacheSize"));
            }
        }
        if (clientOptions.get("userCountry") != null && !Objects.equals(clientOptions.get("userCountry"), "") && clientOptions.get("userCountry") instanceof String) {
            builder = builder.userCountry((String) clientOptions.get("userCountry"));
        }
        if (clientOptions.get("checkForCMP") != null && clientOptions.get("checkForCMP") instanceof Boolean) {
            builder = builder.checkForCMP((Boolean) clientOptions.get("checkForCMP"));
        }
        if (clientOptions.get("purposesForTracking") != null && clientOptions.get("purposesForTracking") instanceof List) {
            builder = builder.purposesForTracking(convertDoubleListToIntegerList((ArrayList<Object>) clientOptions.get("purposesForTracking")));
        }
        if (clientOptions.get("purposesForIdentify") != null && clientOptions.get("purposesForIdentify") instanceof List) {
            builder = builder.purposesForIdentifying(convertDoubleListToIntegerList((ArrayList<Object>) clientOptions.get("purposesForIdentify")));
        }
        if (clientOptions.get("areIdentitiesHashed") != null && clientOptions.get("areIdentitiesHashed") instanceof Boolean) {
            builder = builder.areIdentitiesHashed((Boolean) clientOptions.get("areIdentitiesHashed"));
            if (clientOptions.get("hashIdentities") != null && clientOptions.get("hashIdentities") instanceof Boolean) {
                builder = builder.hashIdentities((Boolean) clientOptions.get("hashIdentities"));
            }
        }
//        if (clientOptions.get("checkZeotapVendorConsent") != null && clientOptions.get("checkZeotapVendorConsent") instanceof Boolean) {
//            builder = builder.checkZeotapVendorConsent((Boolean) clientOptions.get("checkZeotapVendorConsent"));
//        }
        CollectOptions options = builder.build();
        Collect.init(options);
        collect = Collect.getInstance();
    }

    private ArrayList<Integer> convertDoubleListToIntegerList(ArrayList<Object> list) {
        ArrayList<Integer> convertedList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i) instanceof Double) {
                convertedList.add(((Double) list.get(i)).intValue());
            } else if (list.get(i) instanceof Integer) {
                convertedList.add((Integer) (list.get(i)));
            }
        }
        return convertedList;
    }

    @Override
    @NonNull
    public String getName() {
        return NAME;
    }

    @ReactMethod
    public void setEventNameProperties(String eventName) {
        Log.d("teest", collect == null ? "collect in setEventNameProperties, collect is null" : "collect in setEventNameProperties, collect is not null" + collect.getClass().toString());
        if (collect != null) {
            collect.setEventProperties(eventName);
        }
    }

    @ReactMethod
    public void setEventProperties(String eventName, ReadableMap eventProp) {
        Log.d("teest", collect == null ? "collect in setEventProperties, collect is null" : "collect in setEventProperties, collect is not null" + collect.getClass().toString());
        if (collect != null) {
            collect.setEventProperties(eventName, eventProp.toHashMap());
        }
    }

    @ReactMethod
    public void setInstantEventNameProperties(String eventName) {
        Log.d("teest", collect == null ? "collect in setInstantEventNameProperties, collect is null" : "collect in setInstantEventNameProperties, collect is not null" + collect.getClass().toString());
        if (collect != null) {
            collect.setInstantEventProperties(eventName);
        }
    }

    @ReactMethod
    public void setInstantEventProperties(String eventName, ReadableMap eventProp) {
        Log.d("teest", collect == null ? "collect in setInstantEventProperties, collect is null" : "collect in setInstantEventProperties, collect is not null" + collect.getClass().toString());
        if (collect != null) {
            collect.setInstantEventProperties(eventName, eventProp.toHashMap());
        }
    }

    @ReactMethod
    public void setUserIdentities(ReadableMap userProp) {
        if (collect != null) {
            collect.setUserIdentities(userProp.toHashMap());
        }
    }

    @ReactMethod
    public void setUserProperties(ReadableMap userProp) {
        if (collect != null) {
            collect.setUserProperties(userProp.toHashMap());
        }
    }

    @ReactMethod
    public void setPageProperties(ReadableMap properties) {
        Log.d("teest", collect == null ? "collect in setPageProperties, collect is null" : "collect in setPageProperties, collect is not null" + collect.getClass().toString());
        if (collect != null) {
            collect.setPageProperties(properties.toHashMap());
        }
    }

    @ReactMethod
    public void unsetUserIdentities() {
        if(collect != null) {
            collect.unsetUserIdentities();
        }
    }

    @ReactMethod
    public void deInitiate() {
        if(collect != null) {
            collect.pauseCollection();
        }
    }

    @ReactMethod
    public void reInitialize() {
        if(collect != null) {
            collect.resumeCollection();
        }
    }

    @ReactMethod
    public void setConsent(ReadableMap consent) {
        if(collect != null) {
            collect.setConsent(consent.toHashMap());
        }
    }

    @ReactMethod
    public void addAskForConsentListener(Callback cb) {
        if(collect != null) {
            collect.listenToAskForConsent((data) -> cb.invoke());
        }
    }

    @ReactMethod
    public void getZI(Callback cb) {
        if(collect != null) {
            String zi = collect.getZI();
            cb.invoke(zi);
        }
    }

    @ReactMethod
    public void resetZI() {
        if(collect != null) {
            collect.resetZI();
        }
    }
}
